[{{ site_id }}]

user = {{ user }}
group = {{ user }}

listen = /run/php/php{{ php_version }}-fpm-{{ site_id }}.sock

listen.owner = {{ user }}
listen.group = {{ user }}

pm = dynamic
pm.max_children = 50
pm.start_servers = 2
pm.min_spare_servers = 1
pm.max_spare_servers = 3
pm.max_requests = 500

{% for value in php_flag %}
php_flag[{{ value }}] = {{ php_flag[value] }}
{% endfor %}

{% for value in php_admin_value %}
php_admin_value[{{ value }}] = {{ php_admin_value[value] }}
{% endfor %}
